' Floating Point Array Load and Retrieve Macro
'
' ** For 18F PICmicro devices only **
'
';---------------------------------------------------------------------------------------
' The Actual array is build from a single or multiple byte arrays
' If a floating point array of size 20 is required, the byte array must be for times this
' because floating point variables and constants require 4 bytes to hold their value
' So create the array:
'
' Dim MyArray[80] as Byte				' Room for 20 floating point elements. i.e. 20 * 4
'
'
' Syntax:
'
' Retreive from a Floating Point Array
' FloatVar = FloatArray Array Name , Array Index
'
' Load into a Floating Point Array
' FloatArray Array Name , Array Index , FloatVar
'
FloatArray Macro ArrayName, ArrayIndex, ValueToPlace
#if ((PRM_1 != BYTE_ARRAY_PTR) && (PRM_1 != WORD_ARRAY_PTR))
    #error "Parameter 1 must be an Array's Name"
    exitm
#endif
    Lfsr 0,(ArrayName)
#if((PRM_2 == NUM8) || (PRM_2 == NUM16)) 	; Index is a Constant value
    #if (ArrayIndex > 0)
    Movlw (ArrayIndex)                   	; \
    Mullw 4                             	; / Multiply index by four
	Movf PRODL,w                        	; \
    Addwf FSR0L,f                       	;  \ Add it to the value already held in FSR0L\H
    Movf PRODH,w                        	;  /
    Addwfc FSR0H,f                      	; /
    #endif
#endif

#if(PRM_2 == BYTE)                  		; Index is a Byte Variable
	#if((ArrayIndex <= BANKA_END) || (ArrayIndex >= (_RAM_END - (0xFF - BANKA_END))))
		byte_wreg ArrayIndex
    #else
   	    byte_byte ArrayIndex,WREG
    #endif
	Mullw 4           						; Multiply index by four
	Movf PRODL,w                        	; \
    Addwf FSR0L,f                       	;  \ Add it to the value already held in FSR0L\H
    Movf PRODH,w                        	;  /
    Addwfc FSR0H,f                      	; /
#endif

#if((PRM_2 == WORD) || (PRM_2 == DWORD)) 	; Index is a Word Variable
    Asm
    Bcf STATUS,C
	Rlcf ArrayIndex,w
    Movwf FSR0L
	Rlcf ArrayIndex + 1,w
    Movwf FSR0H
    Bcf STATUS,C
    Rlcf FSR0L,f
    Rlcf FSR0H,f
    Endasm
#endif
'
' Load or Retrieve from the array depending on whether the macro has an assignment variable preceeding it
'
#if (FLOATARRAY_RETURN == 0)        		; No assignment variable found, so it is a loading macro
    #if (PRM_COUNT != 3)
         #error "Missing Loading Operand with FloatArray macro"
    #else
    #if((PRM_3 != FLOAT) && (PRM_3 != FNUM))
     	#error "FloatArray macro can only assign a Floating Point Variable or Constant"
    #endif
    #if(PRM_3 == FLOAT)
    Asm
    	movff ValueToPlace,POSTINC0
    	movff ValueToPlace + 1,POSTINC0
    	movff ValueToPlace + 2,POSTINC0
    	movff ValueToPlace + 3,POSTINC0
   	Endasm
    #else
	Asm
    Movlw (ValueToPlace >> 24) & 255
    Movwf POSTINC0
    Movlw (ValueToPlace >> 16) & 255
    Movwf POSTINC0
    Movlw (ValueToPlace >> 8) & 255
    Movwf POSTINC0
    Movlw ValueToPlace & 255
    Movwf INDF0
    Endasm
    #endif
    #endif
#else                                  		; Otherwise it is a retreiving macro
    #if(RETURN_TYPE != FLOAT)
    	#error "FloatArray macro can only return a Floating Point Variable or Constant"
    #endif
	Asm
    movff POSTINC0,RETURN_VAR
    movff POSTINC0,RETURN_VAR + 1
    movff POSTINC0,RETURN_VAR + 2
    movff INDF0,RETURN_VAR + 3
    Endasm
#endif
    Endm


