;
; SPI peripheral macros 
;
;
; SSPSTAT Register 
; Master SPI mode only
;
	Symbol   SMPEND        $80 NO_LIST      ; Input data sample at end of data out             
	Symbol   SMPMID        $00 NO_LIST      ; Input data sample at middle of data out
	Symbol   MODE_00       0 NO_LIST        ; Setting for SPI bus Mode 0,0
	Symbol   MODE_01       1 NO_LIST        ; Setting for SPI bus Mode 0,1
	Symbol   MODE_10       2 NO_LIST        ; Setting for SPI bus Mode 1,0
	Symbol   MODE_11       3 NO_LIST        ; Setting for SPI bus Mode 1,1
	Symbol   MODE_0        0 NO_LIST        ; Setting for SPI bus Mode 0,0
	Symbol   MODE_1        1 NO_LIST        ; Setting for SPI bus Mode 0,1
	Symbol   MODE_2        2 NO_LIST        ; Setting for SPI bus Mode 1,0
	Symbol   MODE_3        3 NO_LIST        ; Setting for SPI bus Mode 1,1
;
; SSPCON1 Register 
;
	Symbol   SSPENB        $20 NO_LIST      ; Enable serial port and configures SCK, SDO, SDI
	Symbol   SPI_FOSC_4    0 NO_LIST        ; SPI Master mode, clock = Fosc/4
	Symbol   SPI_FOSC_16   1 NO_LIST        ; SPI Master mode, clock = Fosc/16
	Symbol   SPI_FOSC_64   2 NO_LIST        ; SPI Master mode, clock = Fosc/64
	Symbol   SPI_FOSC_TMR2 3 NO_LIST        ; SPI Master mode, clock = TMR2 output/2
	Symbol   SLV_SSON      4 NO_LIST        ; SPI Slave mode, /SS pin control enabled
	Symbol   SLV_SSOFF     5 NO_LIST        ; SPI Slave mode, /SS pin control disabled

@Variable SPI#LBL#CNTR = 0					; Used for label jumps with the macros

;-----------------------------------------------------------------------------------
; Macro Name	SPI_OPEN                                        
; Return Value	: None                                            
; Parameters	: SSP1 peripheral setup values                     
; Notes			: Sets up the SSP1 module for master or slave operation                    
;
; Syntax SPI_OPEN SPI_SYNC_MODE , SPI_BUS_MODE , SPI_SMP_PHASE
;
; SPI_SYNC_MODE can be:- SPI_FOSC_4 or SPI_FOSC_16 or SPI_FOSC_64 or SPI_FOSC_TMR2 3 or SLV_SSON or SLV_SSOFF
; SPI_BUS_MODE can be:- MODE_00 or MODE_01 or MODE_10 or MODE_11 or MODE_0 or MODE_1 or MODE_2 or MODE_3
; SPI_SMP_PHASE can be:- SMPEND or SMPMID
;
SPI_OPEN MACRO- SPI_SYNC_MODE , SPI_BUS_MODE , SPI_SMP_PHASE

  	SSPSTAT = SSPSTAT & $3F             ; Power on state 
  	SSPCON1 = $00                		; Power on state
  	#if(SPI_SYNC_MODE != 0)
    	Movlw	(SPI_SYNC_MODE)
  		Iorwf 	SSPCON1,F				; Select serial mode 
  	#endif
    #if(SPI_SMP_PHASE != 0)
    	Movlw	(SPI_SMP_PHASE)
  		Iorwf	SSPSTAT					; Select data input sample phase
	#endif
    
    #if(SPI_BUS_MODE == 0)				; SPI bus mode 0,0
    	Bsf SSPSTAT,CKE       			; Data transmitted on rising edge
  	#endif
    #if(SPI_BUS_MODE == 2)				; SPI bus mode 1,0
    	Bsf SSPSTAT,CKE       			; Data transmitted on falling edge
      	Bsf SSPCON1,CKP       			; Clock idle state high
  	#endif
    #if(SPI_BUS_MODE == 3)				; SPI bus mode 1,1
    	Bsf SSPCON1,CKP       			; Clock idle state high
  	#endif

  	#if(SPI_SYNC_MODE == 4)				; Slave mode with /SS enable
ASM-
NOLIST
        #ifdef TRISF
LIST
        Bsf TRISF,7      				; define /SS pin as input
NOLIST
		#else
LIST
        Bsf TRISA,5      		 		; define /SS pin as input
NOLIST
		#endif
LIST
ENDASM-
  	#endif
  	#if(SPI_SYNC_MODE == 5)             ; Slave mode withouto /SS enable
        #if (_USB == 1)
        Bsf TRISB,1       				; define clock pin as input
        #else
        Bsf TRISC,3						; define clock pin as input
        #endif
      	Bcf SSPSTAT,SMP      			; Must be cleared in slave SPI mode
  	#else                      			; Master mode, define clock pin as output
		#if (_USB == 1)					; Is it a USB capable device ?
        Bcf TRISB,1       				; define clock pin as output
        #else
        Bcf TRISC,3						; define clock pin as output
        #endif
	#endif
    #if (_USB == 1)						; Is it a USB capable device ?
    	TRISC = TRISC & $7F             ; define SDO as output (master or slave)
  		TRISB = TRISB | $01             ; define SDI as input (master or slave)
    #else
  		TRISC = TRISC & $DF             ; Define SDO as output (master or slave)
  		TRISC = TRISC | $10             ; Define SDI as input (master or slave)
  	#endif
    SSPCON1 = SSPCON1 | SSPENB          ; Enable synchronous serial port 
 	ENDM
;-----------------------------------------------------------------------------------
; Macro Name    SPI_CLOSE                                   
; Return Value	: None                                        
; Parameters	: None                                        
; Notes			: Disable the SSP module. 
;				  Pin I/O returns under the control of the port registers. 
;                                 
SPI_CLOSE MACRO-
  	SSPCON1 = SSPCON1 & $DF               ; disable synchronous serial port
    ENDM
;-----------------------------------------------------------------------------------
; Macro Name    SPI_WRITE                                    
; Return Value	:              
; Parameter		: Single 8-bit variable for SPI bus.               
; Description	: Write a single byte (8-bits) to the SPI bus.
;
SPI_WRITE MACRO- SPI_DATA_OUT \BYTE

    #if((PRM_1 == NUM8) || (PRM_1 == NUM16) || (PRM_1 == NUM32))	; Is the parameter a constant value ?
    	#if(SPI_DATA_OUT == 0)					; \
        	Clrf SSPBUF							;  \
        #else									;   \ 
        	#if(SPI_DATA_OUT == 255)			;    \
        		Setf SSPBUF						;     \
        	#else								;     / Write the value to SSPBUF register
        		Movlw (SPI_DATA_OUT)			;    / 
        		Movwf SSPBUF					;   /
        	#endif								;  /
        #endif									; /       
    #endif
        
    #if((PRM_1 == BYTE)	|| (PRM_1 == WORD) || (PRM_1 == DWORD))		; Is the parameter a variable ?
    	#if(SPI_DATA_OUT == WREG)
        	Movwf SSPBUF						; Write WREG to SSPBUF register
        #else									; Otherwise..
        	Byte_Byte SPI_DATA_OUT,SSPBUF		; Write byte to SSPBUF register
    	#endif        
    #endif
	Btfss SSPSTAT,BF							; \ Wait until bus cycle complete
    Bra $ - 2									; /
    #if(SPI_WRITE_RETURN == 1)					; Is there a value to return ?
    	#if (RETURN_TYPE != BYTE)			; Yes. So is it a BYTE variable ?
    		#error "SPI_WRITE can only return a BYTE value" ; No. So issue an error
    	#else
        	#if (RETURN_VAR == WREG)
           		Movf SSPBUF,W
          	#else
           		Byte_Byte SSPBUF,RETURN_VAR
         	#endif
     	#endif
	#endif
    ENDM    
;-----------------------------------------------------------------------------------
; Macro Name	: SPI_READ                                     
; Return Value	: Contents of SSPBUF register                 
; Parameters	: None                                        
; Description	: Read byte\s from the SPI bus.              

SPI_READ MACRO- \BYTE
  	SSPBUF = $FF                  				; Initiate bus cycle
  	Btfss SSPSTAT,BF
  	Bra $ - 2									; Wait until cycle complete
  	#if(SPI_READ_RETURN == 1)
    	#if(RETURN_TYPE == BYTE)
        	#if((RETURN_VAR == WREG) || (RETURN_VAR == SSPBUF))
            	Movf SSPBUF,W
            #else
        		Byte_Byte SSPBUF,RETURN_VAR
            #endif
        #endif
        #if(RETURN_TYPE == WORD)
        	Byte_Byte SSPBUF,RETURN_VAR
            Btfss SSPSTAT,BF
  			Bra $ - 2							; Wait until cycle complete
        	Byte_Byte SSPBUF,RETURN_VAR + 1
        #endif
        #if(RETURN_TYPE == DWORD)
        	Byte_Byte SSPBUF,RETURN_VAR
            Btfss SSPSTAT,BF
  			Bra $ - 2							; Wait until cycle complete
        	Byte_Byte SSPBUF,RETURN_VAR + 1
            Btfss SSPSTAT,BF
  			Bra $ - 2							; Wait until cycle complete
        	Byte_Byte SSPBUF,RETURN_VAR + 2
            Btfss SSPSTAT,BF
  			Bra $ - 2							; Wait until cycle complete
        	Byte_Byte SSPBUF,RETURN_VAR + 3
        #endif
    #endif
    ENDM
;-----------------------------------------------------------------------------------
; Macro Name    SPI_READY                                  
; Return Value	: STATUS byte to indicate if BF = 1           
; Parameters	: None                                        
; Description	: Determine if there is a byte to be read from the SSPBUF register.                   
;
SPI_READY MACRO
	#if(SPI_READY_RETURN == 1)
    	#if (RETURN_TYPE == BYTE)
    		Num_Byte 1,RETURN_VAR
    	#endif
    	#if (RETURN_TYPE == WORD)
    		Num_Word 1,RETURN_VAR
    	#endif
    	#if (RETURN_TYPE == DWORD)
    		Num_Dword 1,RETURN_VAR
    	#endif
        #if(RETURN_TYPE == BIT)
        	Num_Bit 1,RETURN_VAR,RETURN_BITNUM
        #endif
        Btfsc SSPSTAT,BF	 	
        Bra SPI_READY#EX#v(SPI#LBL#CNTR)
    	#if (RETURN_TYPE == BYTE)
    		Num_Byte 0,RETURN_VAR
    	#endif
    	#if (RETURN_TYPE == WORD)
    		Num_Word 0,RETURN_VAR
    	#endif
    	#if (RETURN_TYPE == DWORD)
    		Num_Dword 0,RETURN_VAR
        #endif
        #if(RETURN_TYPE == BIT)
        	Num_Bit 0,RETURN_VAR,RETURN_BITNUM
        #endif
ASM-
SPI_READY#EX#v(SPI#LBL#CNTR)
SPI#LBL#CNTR++
ENDASM-
    #endif
    ENDM
;-----------------------------------------------------------------------------------    
;
; Check if collision occured
;  
;  	#if(SPI_WRITE_RETURN == 1)
;    	#if ((RETURN_TYPE == BYTE) || (RETURN_TYPE == WORD) || (RETURN_TYPE == DWORD))
;    		Num_Byte 1,RETURN_VAR
;    	#endif
;        #if(RETURN_TYPE == BIT)
;        	Num_Bit 1,RETURN_VAR,RETURN_BITNUM
;        #endif
;        Btfss SSPCON1,7	 	
;        Bra SPI_WRITE#EX#v(SPI#LBL#CNTR)
;  		Btfss SSPSTAT,BF						; \ Wait until bus cycle complete
;    	Bra $ - 2								; /
;    	#if ((RETURN_TYPE == BYTE) || (RETURN_TYPE == WORD) || (RETURN_TYPE == DWORD))
;    		Num_Byte 0,RETURN_VAR				; if WCOL bit is not set return non-negative#
;    	#endif
;        #if(RETURN_TYPE == BIT)
;        	Num_Bit 0,RETURN_VAR,RETURN_BITNUM
;        #endif
;    #endif

    


