'
' TIMER2 MACROS
'
' For 16-bit core (18F) devices using version 3.1 onwards of the PROTON+ Compiler
'  
' Interrupt bit mask to be 'anded' with the other configuration masks and
' passed as the PARAM_CONFIG parameter to the 'open' macros.
'
	Symbol T2_INT_OFF   = %01111111   		' Interrupts disabled
	Symbol T2_INT_ON    = %11111111   		' Interrupts enabled
'
' TIMER2 configuration masks - to be 'anded' together and passed to the OPEN_TIMER2 macro
'
' Prescale Value
'
	Symbol T2_PS_1_1       = %11111100   	' Prescale 1:1
	Symbol T2_PS_1_4       = %11111101   	' Prescale 1:4
	Symbol T2_PS_1_16      = %11111110   	' Prescale 1:16
'
' Postscale Value
'
	Symbol T2_POST_1_1     = %10000111   	' Postscaler 1:1
	Symbol T2_POST_1_2     = %10001111   	' Postscaler 1:2
	Symbol T2_POST_1_3     = %10010111   	' Postscaler 1:3
	Symbol T2_POST_1_4     = %10011111   	' Postscaler 1:4
	Symbol T2_POST_1_5     = %10100111   	' Postscaler 1:5
	Symbol T2_POST_1_6     = %10101111   	' Postscaler 1:6
	Symbol T2_POST_1_7     = %10110111   	' Postscaler 1:7
	Symbol T2_POST_1_8     = %10111111   	' Postscaler 1:8
	Symbol T2_POST_1_9     = %11000111   	' Postscaler 1:9
	Symbol T2_POST_1_10    = %11001111   	' Postscaler 1:10
	Symbol T2_POST_1_11    = %11010111   	' Postscaler 1:11
	Symbol T2_POST_1_12    = %11011111   	' Postscaler 1:12
	Symbol T2_POST_1_13    = %11100111   	' Postscaler 1:13
	Symbol T2_POST_1_14    = %11101111   	' Postscaler 1:14
	Symbol T2_POST_1_15    = %11110111   	' Postscaler 1:15
	Symbol T2_POST_1_16    = %11111111   	' Postscaler 1:16
'---------------------------------------------------------------------------------
' Macro Name	: OPEN_TIMER2                                     
' Return Value	: None                                           
' Parameters	: Bit definitions to configure Timer2    
' Notes			: Reset the Timer2 regs to the POR state and configures the interrupt, clock source, edge and prescaler.                    
' 				: The bit definitions for PARAM_CONFIG can be found at the top of this file.                          
'
OPEN_TIMER2 MACRO PARAM_CONFIG		
  	#if(PRM_COUNT != 1)
    	#error "OPEN_TIMER2 Macro requires a parameter"
    	EXITM
    #endif
    #if((PRM_1 != NUM8) && (PRM_1 != NUM16) && (PRM_1 != NUM32))
    	#error "OPEN_TIMER2 Macro parameter must be a constant value"
    	EXITM
    #endif
    #if((0XFB & PARAM_CONFIG) == 0)
    	Clrf T2CON
    #else
    	Movlw (0XFB & PARAM_CONFIG)	
  		Movwf T2CON					' Configure Timer2, but don't start it yet
  	#endif  
    TMR2 = 0                		' Reset Timer2 to 0X00	
    Bcf PIR1,TMR2IF      			' Clear Timer2 overflow flag
  	#if(PARAM_CONFIG & 0X80)        ' If interrupts enabled
    	Bsf PIE1,TMR2IE    			' Enable Timer2 overflow interrupt
    #else
    	Bcf PIE1,TMR2IE
  	#endif
  	Bsf T2CON,TMR2ON				' Enable Timer2
	ENDM  
'---------------------------------------------------------------------------------
' Macro Name 	: CLOSE_TIMER2                                    
' Return Value	: None                                           
' Parameters	: None                                           
' Description	: Disable the Timer2 interrupt and stop Timer2    
'
CLOSE_TIMER2 MACRO
    Bcf T2CON,TMR1ON				' Disable Timer2
  	Bcf PIE1,TMR2IE					' Disable Timer2 overflow interrupts
    ENDM       
'---------------------------------------------------------------------------------     
' Macro Name  	: READ_TIMER2                                     
' Return Value	: Timer2 8-bit value                       
' Parameters	: None                                           
' Description	: Read an 8-bit value from Timer2.                                        
'
READ_TIMER2 MACRO
    #if(READ_TIMER2_RETURN == 1)
        #if(RETURN_TYPE == BYTE)
        	#if(RETURN_VAR == WREG)
                Movfw TMR2
            #else
                Byte_Byte TMR2,RETURN_VAR  	' Copy Timer2
  			#endif
        #endif
        #if(RETURN_TYPE == WORD)
        	Byte_Word TMR2,RETURN_VAR  		' Copy Timer2
        #endif
        #if(RETURN_TYPE == DWORD)
        	Byte_Dword TMR2,RETURN_VAR  	' Copy Timer2
        #endif
    #endif
  	ENDM 
'---------------------------------------------------------------------------------
' Macro Name  	: WRITE_TIMER2                                    
' Return Value	: None                                           
' Parameters	: Value to write to Timer2                  
' Description	: Write an 8-bit value to Timer2                                           
'
WRITE_TIMER2 MACRO PARAM_TIMER

	#if(PRM_COUNT != 1)
    	#error "WRITE_TIMER2 Macro requires a parameter: the value placed into TIMER2"
    	EXITM
    #endif
    
    #if((PRM_1 == NUM8) || (PRM_1 == NUM16) || (PRM_1 == NUM32))
    	#if(PARAM_TIMER == 0)
        	Clrf TMR2
        #else
        	Movlw (PARAM_TIMER & 255)
        	Movwf TMR2
        #endif
    #endif
    #if((PRM_1 == BYTE) || (PRM_1 == WORD) || (PRM_1 == DWORD))
   		#if(PARAM_TIMER == WREG)
     		Movwf TMR2
    	#else
      		Byte_Byte PARAM_TIMER,TMR2
    	#endif
    #endif
    ENDM  
    