'
' TIMER4 MACROS
'
' For 16-bit core (18F) devices using version 3.1 onwards of the PROTON+ Compiler
'
' Interrupt bit mask to be 'anded' with the other configuration masks and
' passed as the PARAM_CONFIG parameter to the 'open' macros.
'
	Symbol T4_INT_OFF   = %01111111   			' Interrupts disabled
	Symbol T4_INT_ON    = %11111111   			' Interrupts enabled
' 
' TIMER4 configuration masks - to be 'anded' together and passed to the 'open' macro
'
' Prescale Value
'
	Symbol T4_PS_1_1       = %11111100   		' Prescale 1:1
	Symbol T4_PS_1_4       = %11111101   		' Prescale 1:4
	Symbol T4_PS_1_16      = %11111111   		' Prescale 1:16
'
' Postscale Value
'
	Symbol T4_POST_1_1     = %10000111   		' Postscaler 1:1
	Symbol T4_POST_1_2     = %10001111   		' Postscaler 1:2
	Symbol T4_POST_1_3     = %10010111   		' Postscaler 1:3
	Symbol T4_POST_1_4     = %10011111   		' Postscaler 1:4
	Symbol T4_POST_1_5     = %10100111   		' Postscaler 1:5
	Symbol T4_POST_1_6     = %10101111   		' Postscaler 1:6
	Symbol T4_POST_1_7     = %10110111   		' Postscaler 1:7
	Symbol T4_POST_1_8     = %10111111   		' Postscaler 1:8
	Symbol T4_POST_1_9     = %11000111   		' Postscaler 1:9
	Symbol T4_POST_1_10    = %11001111   		' Postscaler 1:10
	Symbol T4_POST_1_11    = %11010111   		' Postscaler 1:11
	Symbol T4_POST_1_12    = %11011111   		' Postscaler 1:12
	Symbol T4_POST_1_13    = %11100111   		' Postscaler 1:13
	Symbol T4_POST_1_14    = %11101111   		' Postscaler 1:14
	Symbol T4_POST_1_15    = %11110111   		' Postscaler 1:15
	Symbol T4_POST_1_16    = %11111111   		' Postscaler 1:16
'---------------------------------------------------------------------------------
' Macro Name	: OPEN_TIMER4                                     
' Return Value	: None                                           
' Parameters	: Bit definitions to configure Timer4    
' Notes			: Reset the Timer4 regs to the POR state and configures the interrupt, clock source, edge and prescaler.                    
' 				: The bit definitions for PARAM_CONFIG can be found at the top of this file.                          
'
OPEN_TIMER4 MACRO PARAM_CONFIG		
  	#if(PRM_COUNT != 1)
    	#error "OPEN_TIMER4 Macro requires a parameter"
    	EXITM
    #endif
    #if((PRM_1 != NUM8) && (PRM_1 != NUM16) && (PRM_1 != NUM32))
    	#error "OPEN_TIMER4 Macro parameter must be a constant value"
    	EXITM
    #endif
    #if((0XFB & PARAM_CONFIG) == 0)
    	Clrf T4CON
    #else
    	Movlw (0XFB & PARAM_CONFIG)	
  		Movwf T4CON					' Configure Timer4, but don't start it yet
  	#endif  
    TMR4 = 0                		' Reset Timer4 to 0X00	
    Bcf PIR3,TMR4IF      			' Clear Timer4 overflow flag
  	#if(PARAM_CONFIG & 0X80)        ' If interrupts enabled
    	Bsf PIE3,TMR4IE    			' Enable Timer4 overflow interrupt
    #else
    	Bcf PIE3,TMR4IE
  	#endif
  	Bsf T4CON,TMR4ON				' Enable Timer4
	ENDM   
'---------------------------------------------------------------------------------
' Macro Name 	: CLOSE_TIMER4                                    
' Return Value	: None                                           
' Parameters	: None                                           
' Description	: Disable the Timer4 interrupt and stop Timer4    
'
CLOSE_TIMER4 MACRO
    Bcf T4CON,TMR1ON				' Disable Timer4
  	Bcf PIE3,TMR4IE					' Disable Timer4 overflow interrupts
    ENDM       
'---------------------------------------------------------------------------------     
' Macro Name  	: READ_TIMER4                                     
' Return Value	: Timer4 8-bit value                       
' Parameters	: None                                           
' Description	: Read an 8-bit value from Timer4.                                        
'
READ_TIMER4 MACRO
    #if(READ_TIMER4_RETURN == 1)
        #if(RETURN_TYPE == BYTE)
        	#if(RETURN_VAR == WREG)
                Movfw TMR4
            #else
                Byte_Byte TMR4,RETURN_VAR  	' Copy Timer4
  			#endif
        #endif
        #if(RETURN_TYPE == WORD)
        	Byte_Word TMR4,RETURN_VAR  		' Copy Timer4
        #endif
        #if(RETURN_TYPE == DWORD)
        	Byte_Dword TMR4,RETURN_VAR  	' Copy Timer4
        #endif
    #endif
  	ENDM 
'---------------------------------------------------------------------------------
' Macro Name  	: WRITE_TIMER4                                    
' Return Value	: None                                           
' Parameters	: Value to write to Timer4                  
' Description	: Write an 8-bit value to Timer4                                           
'
WRITE_TIMER4 MACRO PARAM_TIMER

	#if(PRM_COUNT != 1)
    	#error "WRITE_TIMER4 Macro requires a parameter: the value placed into TIMER4"
    	EXITM
    #endif
    
    #if((PRM_1 == NUM8) || (PRM_1 == NUM16) || (PRM_1 == NUM32))
    	#if(PARAM_TIMER == 0)
        	Clrf TMR4
        #else
        	Movlw (PARAM_TIMER & 255)
        	Movwf TMR4
        #endif
    #endif
    #if((PRM_1 == BYTE) || (PRM_1 == WORD) || (PRM_1 == DWORD))
   		#if(PARAM_TIMER == WREG)
     		Movwf TMR4
    	#else
      		Byte_Byte PARAM_TIMER,TMR4
    	#endif
    #endif
    ENDM 
'---------------------------------------------------------------------------------     
   